/*
   This is a part of the Niver's Settings IPC LiteStep Module Source code.

   Copyright (C) 2003 Niversoft 
   http//niversoft.dyndns.org
   info@niversoft.dyndns.org

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-------------------------------------------------------------------------------------

#include "windows.h"
#include "settingsipc_client.h"
#include <stdio.h>

settingsipc_client::settingsipc_client()
{
   m_hFree = CreateMutex(NULL, FALSE, MUTEXNAME);
   m_hExec = CreateEvent(NULL, FALSE, FALSE, EVENT_EXEC);
   m_hDone = CreateEvent(NULL, FALSE, FALSE, EVENT_DONE);
}

settingsipc_client * settingsipc_client::GetInstance()
{
   static settingsipc_client client;
   return &client;
}

bool settingsipc_client::Connect(eAction Msg)
{
   WaitForSingleObject(m_hFree, INFINITE);

   hMap = OpenFileMapping(FILE_MAP_WRITE, FALSE, SHAREDMEMNAME);
   if (!hMap) 
   {      
      ReleaseMutex(m_hFree);
      return false;
   }
   
   pMsg = (TSharedMemory*)MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, sizeof(TSharedMemory));
   if (!pMsg)
   {
      CloseHandle(pMsg);
      ReleaseMutex(m_hFree);
      return false;
   }
   
   pMsg->Action = Msg;
   
   return true;
}

void settingsipc_client::Disconnect()
{
   UnmapViewOfFile(pMsg);
   CloseHandle(hMap);   

   ReleaseMutex(m_hFree);
}

bool settingsipc_client::Execute()
{
   SetEvent(m_hExec);
   return WaitForSingleObject(m_hDone, TIMEOUT) == WAIT_OBJECT_0;  
}

long settingsipc_client::SendMessage(eAction Msg, LPCSTR key, long def)
{
   if (!Connect(Msg)) return 0;

   pMsg->nDefaultValue = def;
   pMsg->nRetValue = def;
   pMsg->Action = Msg;
   strcpy(pMsg->szKeyName, key);

   Execute();
   
   long ret = pMsg->nRetValue;
   
   Disconnect();

   return ret;
}

BOOL settingsipc_client::SendMessage(eAction Msg, LPCSTR key, LPSTR buf, LPCSTR def, int maxLen)
{
   if (!Connect(Msg)) return 0;
   
   pMsg->nMaxLength = maxLen;
   strcpy(pMsg->szDefaultValue, def);   
   strcpy(pMsg->szKeyName, key);

   strcpy(buf, Execute() ? pMsg->szRetValue : pMsg->szDefaultValue);
   long ret = pMsg->nRetValue;

   Disconnect();
   return ret;
}

HWND settingsipc_client::SendMessage(eAction Msg)
{
   if (!Connect(Msg)) return NULL;   
   HWND ret = Execute() ? pMsg->hWnd : NULL;
   Disconnect();
   return ret;
}

bool settingsipc_client::IsServerPresent()
{   
   if (!GetInstance()->Connect(eaNoAction)) return false;
   bool ret = GetInstance()->Execute();
   GetInstance()->Disconnect();
   return ret;
}


int settingsipc_client::GetRCInt(LPCSTR lpKeyName, int nDefault)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCInt, lpKeyName, nDefault);
}

BOOL settingsipc_client::GetRCBool(LPCSTR lpKeyName, BOOL ifFound)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCBool, lpKeyName, ifFound);
}

BOOL settingsipc_client::GetRCBoolDef(LPCSTR lpKeyName, BOOL bDefault)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCBoolDef, lpKeyName, bDefault);
}

COLORREF settingsipc_client::GetRCColor(LPCSTR lpKeyName, COLORREF colDef)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCColor, lpKeyName, colDef);
}

BOOL settingsipc_client::GetRCString(LPCSTR lpKeyName, LPSTR value, LPCSTR defStr, int maxLen)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCString, lpKeyName, value, defStr, maxLen);
}

BOOL settingsipc_client::GetRCLine( LPCSTR lpKeyName, LPSTR value, UINT maxLen, LPCSTR defStr)
{
   return (BOOL)GetInstance()->SendMessage(eaGetRCLine, lpKeyName, value, defStr, maxLen);
}

HWND settingsipc_client::GetLitestepWnd()
{
   return GetInstance()->SendMessage(eaGetLitestepWnd);
}